!*************************************************************************************************
! output file maker (for point simulation)
!*************************************************************************************************
SUBROUTINE output_point &
   (year, dom, doy, counter, &
   tmp_air, tmp_soil, cloud, prec, humid, wind, &
   W_sat, &
   co2atm, ap, vp, vpd, dnsa, vp_sat, slope_vps, &
   sl_hgt, dlen, rad, par, par_grass, &
   albedo_soil, albedo_veg, radnet_soil, radnet_veg, &
   mort_regu3, gdd, moist_factor, fire_factor, &
   pool_w1, pool_w2, pool_snow, canopy_cond, &
   pool_litter, pool_som_int, pool_som_slow, &
   stat_water, eK, lue, co2cmp, psat, &
   pft, age, height, bole, dbh_heartwood, dbh_sapwood, crown_diameter, crown_area, la, &
   mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
   bole_x, bole_y, crown_x, crown_y, tree_exist, height_limit, radius_limit,&
   type_grass, gmass_leaf, gmass_root, gmass_available, gmass_stock, &
   lai_max_wood, lai_max_grass, lai_mean_wood, lai_mean_grass, lai_grass, lai_opt_grass, &
   pft_exist, biome, C3g_no, C4g_no, &
   phenology, pheno_clo, pheno_cls, pheno_gdd, pheno_ncd, &
   c_emission, c_uptake, gpp, npp, gpp_annual, npp_annual, mort_regu1, &
   hydRecord_preci, hydRecord_runoff, hydRecord_ic, hydRecord_ev, hydRecord_tr)

USE data_structure
implicit none

!loop counters
integer,intent(IN)::year     !loop counter for year (1-Simulation_year)
integer,intent(IN)::dom      !loop counter for day of month (1-31)
integer,intent(IN)::doy      !day of the year (1-365)
integer,intent(IN)::counter  !simulation day (1-Simulation_year*365)

!Climate data
real,intent(IN)::tmp_air     !2m air temperature (Celcius)
real,intent(IN)::tmp_soil    !soil temperature (Celcius)
real,intent(IN)::cloud       !total cloudiness (fraction)
real,intent(IN)::prec        !precipitation (mm day-1)
real,intent(IN)::humid       !specific humidity (kg kg-1)
real,intent(IN)::wind        !wind  velocity (m s-1)

!Location data
real W_sat        !saturate point   (m3/m3, 0.0 -> 1.0)

!air related
real,intent(IN)::co2atm    !ambient (canopy) CO2 concentration (ppmv)
real,intent(IN)::ap        !air pressure (hPa)
real,intent(IN)::vp        !vapour pressure (hPa)
real,intent(IN)::vpd       !vapour pressure deficit (hPa)
real,intent(IN)::dnsa      !density of air (kg m-3)
real,intent(IN)::vp_sat    !saturation vapour pressure (hPa)
real,intent(IN)::slope_vps !slope of saturation vapour pressure related to temp (hPa deg C-1)

!radiation related
real,intent(IN)::sl_hgt       !solar hight at midday (degree)
real,intent(IN)::dlen         !day length (hour)
real,intent(IN)::rad          !global radiation at the ground surface (W/m2)
real,intent(IN)::par          !par of mid-day (micro mol photon m-2 s-1)
real,intent(IN)::par_grass    !par on top of the grass layer (micro mol photon m-2 s-1)

real,intent(IN)::albedo_soil  !soil surface albedo
real,intent(IN)::albedo_veg   !vegetation area albedo
real,intent(IN)::radnet_soil  !soil surface net radiation, short wave (W m-2)
real,intent(IN)::radnet_veg   !canopy net radiation, short wave (W m-2)

!climate related recorder
real,intent(IN)::mort_regu3   !heat stress for each year for mortality calc.
real,intent(IN)::gdd          !growing-degree-day-sum of 5 Celcius base
real,intent(IN)::moist_factor !adjusted moisture factor for fire spread
real,intent(IN)::fire_factor  !factor for fire probability calculation

!hydrogy related
real,intent(IN)::pool_w1      !soil water content of soil layer 1 (mm)
real,intent(IN)::pool_w2      !soil water content of soil layer 2 (mm)
real,intent(IN)::pool_snow    !water equivalent snow depth (mm)
real,intent(IN)::canopy_cond  !monthly stomatal conductance   (mol H2O m-2 s-1)

!soil carbon (s.o.m.; soil organic matter)
real,intent(IN)::pool_litter   !litter pool in a stand (g drymatter / stand)
real,intent(IN)::pool_som_int  !pool of s.o.m. with intermediated decomposition rate (g dm / stand)
real,intent(IN)::pool_som_slow !pool of s.o.m. with slow decomposition rate (g dm / stand)

!photosynthesis related
real,dimension(PFT_no)::stat_water   !state of water satisfactory (0.0-1.0)
real,dimension(PFT_no)::eK           !light attenuation coefficient, no dimension
real,dimension(PFT_no)::lue          !monthly quantum yield        (mol CO2 mol photon-1)
real,dimension(PFT_no)::co2cmp       !CO2 compensation point       (ppmv)
real,dimension(PFT_no)::psat         !light-saturated photosynthesis rate (micro mol CO2 m-2 s-1)

!individual charactristics of woody PFT
integer,dimension(1:Max_no)::pft            !Plant Functional Type (1-PFT_no)
integer,dimension(1:Max_no)::age            !tree age (year)
integer,dimension(1:Max_no)::height         !tree height from 1.3m above ground (STEP)
integer,dimension(1:Max_no)::bole           !bole length from 1.3m above ground (STEP)
real   ,dimension(1:Max_no)::dbh_heartwood  !heartwood diameter at 1.3m   (m)
real   ,dimension(1:Max_no)::dbh_sapwood    !sapwood   diameter at 1.3m   (m)
real   ,dimension(1:Max_no)::crown_diameter !crown diameter (m)
real   ,dimension(1:Max_no)::crown_area     !crown area     (m2)
real   ,dimension(1:Max_no)::la             !leaf area      (m2)
real   ,dimension(1:Max_no)::mass_leaf      !biomass of foliage (g)
real   ,dimension(1:Max_no)::mass_trunk     !biomass of stem    (g)
real   ,dimension(1:Max_no)::mass_root      !biomass of root    (g)
real   ,dimension(1:Max_no)::mass_stock     !biomass for stock  (g)
real   ,dimension(1:Max_no)::mass_available !biomass available  (g)
real   ,dimension(1:Max_no)::bole_x         !x_location in forest stand (m)
real   ,dimension(1:Max_no)::bole_y         !y_location in forest stand (m)
real   ,dimension(1:Max_no)::crown_x        !x_location of crown (m)
real   ,dimension(1:Max_no)::crown_y        !y_location of crown (m)
logical,dimension(1:Max_no)::tree_exist     !tree exist flag
integer,dimension(1:Max_no)::height_limit   !maximum tree height that proximate trees permit
real   ,dimension(1:Max_no)::radius_limit   !maximum canopy radius that proximate trees permit

!grid charactristics of herbaceous PFT
integer type_grass           !grass type priority in this girid (3:C3 type, 4:C4 type)

real,dimension(2)::gmass_leaf      !c3/c4 herbaceous foliage   biomass in a unit area (g/m2)
real,dimension(2)::gmass_root      !c3/c4 herbaceous root      biomass in a unit area (g/m2)
real,dimension(2)::gmass_available !c3/c4 herbaceous available biomass in a unit area (g/m2)
real,dimension(2)::gmass_stock     !c3/c4 herbaceous stock     biomass in a unit area (g/m2)
real             ::lai_max_wood    !Wood  LAI @ annual maximum  (m2/m2)
real             ::lai_max_grass   !Grass LAI @ annual maximum  (m2/m2)
real             ::lai_mean_wood   !Wood  LAI @ annual maximum  (m2/m2)
real             ::lai_mean_grass  !Grass LAI @ annual maximum  (m2/m2)
real,dimension(2)::lai_grass       !c3/c4 leaf area index of grass (m2/m2)
real,dimension(2)::lai_opt_grass   !c3/c4 optimum LAI for grass (m2/m2)

!biome stat
logical,dimension(PFT_no)::pft_exist !flag: PFT exist or not (.TRUE. or .FALSE.)
integer                  ::biome     !biome type number (see subroutine biome_type for definition)

!allocated number of grass PFTs
integer C3g_no !PFT number of C3 grass
integer C4g_no !PFT number of C4 grass

!phenology related
logical,dimension(PFT_no)::phenology !Phenological status (T:foliation, F:dormance)
integer,dimension(PFT_no)::pheno_clo !counter number of last leaf onset
integer,dimension(PFT_no)::pheno_cls !counter number of last leaf shedding
real                     ::pheno_gdd !GDD on 5 degree base
                                     !(reset on 1 Jan for N-hemisphere, 1 July for S-hemisphere)
                                     !
integer                  ::pheno_ncd !Number of chilling day, defined as number of days on which
                                     !the daily mean air temperature is below 5 Cecius degree
                                     !(reset on 1 Nov for N-hemisphere, 1 May for S-hemisphere)
                                     !

!carbon flux recorder
real                  c_emission     !monthly carbon emission (C g / month stand)
real                  c_uptake       !monthly carbon uptake   (C g / month stand)
real,dimension(PFT_no)::gpp          !GPP for each pft (g dm / stand / day)
real,dimension(PFT_no)::npp          !NPP for each pft (g dm / stand / day)
real,dimension(PFT_no)::gpp_annual   !GPP for each pft (g dm / stand / year)
real,dimension(PFT_no)::npp_annual   !NPP for each pft (g dm / stand / year)
real,dimension(1:Max_no)::mort_regu1 !NPP annual (g dm / individual)

!hydrogy record
real hydRecord_preci  !sum of precipitation      (mm/month)
real hydRecord_runoff !sum of runoff water       (mm/month)
real hydRecord_ic     !sum of interception water (mm/month)
real hydRecord_ev     !sum of evaporated water   (mm/month)
real hydRecord_tr     !sum of tranpirated water  (mm/month)

!Daily output (on each specifyed year)
IF (year <= 5 .or. year==10 .or. year==50 .or. year==150) then
     
     Call output_climate &
          (year, doy, tmp_air, tmp_soil, cloud, prec, humid, wind )
     
     Call output_air &
          (year, doy, ap, vp, vpd, dnsa, vp_sat, slope_vps)
     
     Call output_radiation &
          (year, doy, sl_hgt, dlen, rad, par, par_grass)
     
     Call output_netradiation &
          (year, doy, albedo_soil, albedo_veg, &
           radnet_soil, radnet_veg)
     
     Call output_water &
          (year, doy, pool_snow, pool_w1, pool_w2, W_sat)
     
     Call output_grass &
          (year, doy, phenology, gpp, npp, C3g_no, C4g_no, &
          pool_litter, pool_som_int, pool_som_slow, &
          type_grass, gmass_leaf, gmass_root, gmass_available, &
          gmass_stock, lai_grass, lai_opt_grass)
     
END IF

!Monthly output
IF ( dom == Day_in_month(Month(doy)) ) then
     Call output_cflux &
          (year, doy, c_uptake, c_emission, pool_litter, pool_som_int, pool_som_slow, &
           mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
           gmass_leaf, gmass_root, gmass_available, gmass_stock, type_grass)
END IF

!Annualy output
IF (doy == 365) then
     Call output_annual &
        (year, gpp_annual, npp_annual, &
        mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
        dbh_heartwood, dbh_sapwood, tree_exist, &
        gmass_leaf, gmass_root, gmass_stock, gmass_available, &
        type_grass, biome, &
        lai_max_wood, lai_max_grass, lai_mean_wood, lai_mean_grass, &
        pool_litter, pool_som_int, pool_som_slow, &
        la, lai_grass, canopy_cond)
    
    Call output_forest &
        (year, height, bole, dbh_heartwood, dbh_sapwood, crown_diameter, &
        bole_x, bole_y, crown_x, crown_y, pft, tree_exist)
    
    Call output_individual &
       ( year, height(1), bole(1), dbh_heartwood(1), dbh_sapwood(1), crown_diameter(1), &
       la(1), mass_leaf(1), mass_trunk(1), mass_root(1), age(1), pft(1) )
    
END IF

IF (doy == 200) then
     Call output_biomass &
        (mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, tree_exist, pft, &
        gmass_leaf, gmass_root, gmass_stock, gmass_available, type_grass, C3g_no, C4g_no)
END IF


END SUBROUTINE output_point



!**************************************************************************************************
! Output individual properties
!**************************************************************************************************
SUBROUTINE output_individual &
      (year, height, bole, dbh_heartwood, dbh_sapwood, crown_diameter, &
      la, mass_leaf, mass_trunk, mass_root, age, pft)

USE data_structure
implicit none

!Define arguments

integer,intent(IN)::year           !year (1-Simulation_year)
integer,intent(IN)::age            !tree age (year)
integer,intent(IN)::height         !tree height from 1.3m above ground (STEP)
integer,intent(IN)::bole           !bole length (STEP)
integer,intent(IN)::pft            !Plant Functional Type (1-PFT_no)
real   ,intent(IN)::dbh_heartwood  !heartwood diameter at 1.3m   (m)
real   ,intent(IN)::dbh_sapwood    !sapwood diameter at 1.3m   (m)
real   ,intent(IN)::crown_diameter !crown diameter (m)
real   ,intent(IN)::la             !leaf area (m2)
real   ,intent(IN)::mass_leaf      !biomass of foliage (g)
real   ,intent(IN)::mass_trunk     !biomass of stem (g)
real   ,intent(IN)::mass_root      !biomass of root (g)

!Write title
if (year == 1) then
write ( File_no(11),* ) ' Yr age he(m) bo(m)  dia_H  dia_S  dia_C  LA    mas_F  mas_S  mas_R'
endif

!Write data
write (File_no(11),'(2i4, 2f6.1, 2f7.3, 1f7.2, 2f6.1, 3f7.1)') &
   year             ,&   !integer
   age              ,&   !integer
   height*STEP+1.3  ,&   !real
   bole*STEP+1.3    ,&   !real
   dbh_heartwood    ,&   !real
   dbh_sapwood      ,&   !real
   crown_diameter   ,&   !real
   la               ,&   !real
   mass_leaf/1000   ,&   !real
   mass_trunk/1000  ,&   !real
   mass_root/1000        !real

END SUBROUTINE output_individual



!**************************************************************************************************
! Forest status at the every 31 December
!**************************************************************************************************
SUBROUTINE output_annual &
           (year, gpp_annual, npp_annual, &
           mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
           dbh_heartwood, dbh_sapwood, tree_exist, &
           gmass_leaf, gmass_root, gmass_stock, gmass_available, &
           type_grass, biome, &
           lai_max_wood, lai_max_grass, lai_mean_wood, lai_mean_grass, &
           pool_litter, pool_som_int, pool_som_slow, &
           la, lai_grass, canopy_cond)

USE data_structure
implicit none

!*** Define arguments ***
integer                 ,intent(IN)::year       !year

real,dimension(PFT_no)     ,intent(IN)::gpp_annual !GPP for each pft (g dm / stand / year)
real,dimension(PFT_no)     ,intent(IN)::npp_annual !NPP for each pft (g dm / stand / year)

real   ,dimension(1:Max_no),intent(IN)::mass_leaf      !biomass of foliage (g)
real   ,dimension(1:Max_no),intent(IN)::mass_trunk     !biomass of stem    (g)
real   ,dimension(1:Max_no),intent(IN)::mass_root      !biomass of root    (g)
real   ,dimension(1:Max_no),intent(IN)::mass_stock     !biomass for stock  (g)
real   ,dimension(1:Max_no),intent(IN)::mass_available !biomass available  (g)
real   ,dimension(1:Max_no),intent(IN)::dbh_heartwood  !heartwood diameter at 1.3m   (m)
real   ,dimension(1:Max_no),intent(IN)::dbh_sapwood    !sapwood   diameter at 1.3m   (m)
logical,dimension(1:Max_no),intent(IN)::tree_exist     !tree exist flag

real,dimension(1:2),intent(IN)::gmass_leaf      !grass foliage    biomass (g/m2)
real,dimension(1:2),intent(IN)::gmass_root      !grass root       biomass (g/m2)
real,dimension(1:2),intent(IN)::gmass_stock     !grass stock      biomass (g/m2)
real,dimension(1:2),intent(IN)::gmass_available !grass available  biomass (g/m2)
integer            ,intent(IN)::type_grass      !grass type priority (3:C3, 4:C4)

real,intent(IN)::lai_max_wood          !Wood  LAI @ annual maximum  (m2/m2)
real,intent(IN)::lai_max_grass         !Grass LAI @ annual maximum  (m2/m2)
real,intent(IN)::lai_mean_wood         !Wood  LAI @ annual mean     (m2/m2)
real,intent(IN)::lai_mean_grass        !Grass LAI @ annual mean     (m2/m2)

real,intent(IN)::pool_litter   !litter pool in a stand (g dm / stand)
real,intent(IN)::pool_som_int  !pool of s.o.m. with intermediated decomposition rate (g dm / stand)
real,intent(IN)::pool_som_slow !pool of s.o.m. with slow decomposition rate (g dm / stand)

real,dimension(1:Max_no),intent(IN)::la         !leaf area      (m2)
real,dimension(2)       ,intent(IN)::lai_grass  !c3/c4 leaf area index of grass (m2/m2)

real,intent(IN)::canopy_cond  !monthly stomatal conductance   (mol H2O m-2 s-1)

integer,intent(IN)::biome !biome type number (see subroutine biome_type for definition)

!*** Define local variables ***
real    mass_wood      !wood  biomass (g C / m2)
real    mass_grass     !grass biomass (g C / m2)
real    mass_soil      !soil  drymass (g C / m2)
real    gpp, npp       !gpp and npp (gC/m2/year)
real    density        !tree density / above 5cm dbh (N/ha)
real    mass_mean      !mean biomass of tree / above 5cm dbh (kg)
integer i, j           !for temporal usage

!*** prepare output variables ***

   !mass wood
mass_wood  = sum(mass_leaf)+sum(mass_trunk)+sum(mass_root)+sum(mass_stock)+sum(mass_available)
mass_wood  = mass_wood * C_in_drymass / Max_loc / Max_loc /100.0

   !mass grass
if (type_grass==3) then
 mass_grass = &
  0.9*(gmass_leaf(1)+gmass_root(1)+gmass_stock(1)+gmass_available(1))+ &
  0.1*(gmass_leaf(2)+gmass_root(2)+gmass_stock(2)+gmass_available(2))
else
 mass_grass = &
  0.1*(gmass_leaf(1)+gmass_root(1)+gmass_stock(1)+gmass_available(1))+ &
  0.9*(gmass_leaf(2)+gmass_root(2)+gmass_stock(2)+gmass_available(2))
endif
mass_grass = mass_grass * C_in_drymass / 100.0

   !mass soil
mass_soil = pool_litter + pool_som_int + pool_som_slow
mass_soil = mass_soil * C_in_drymass / Max_loc / Max_loc / 100.0

   !gpp
gpp  = sum(gpp_annual) * C_in_drymass / Max_loc / Max_loc / 100.0

   !npp
npp  = sum(npp_annual) * C_in_drymass / Max_loc / Max_loc / 100.0

   !density and mass_mean
j = 0
mass_mean=0.0
do i=1, Max_no
   if ( tree_exist(i)                        ) then
   if ( dbh_heartwood(i) + dbh_sapwood(i) > 0.05 ) then
      j = j+1
      mass_mean = mass_mean + &
                  mass_leaf(i) + mass_trunk(i) + mass_root(i) + mass_stock(i) + mass_available(i)
   endif
   endif
end do
density = real(j) * ( (100.0/Max_loc)**2 )
mass_mean = mass_mean / real(max(1,j)) / 1000.0

!*** write title and data ***
   if (year == 1) then
      write (File_no(2), *) &
      ' wood  grass   soil   gpp   npp  LAImax_w&g  LAImen_w&g  ccon  type F_tree'
      write (File_no(2), *) &
      ' |----(MgC/ha)-----| (MgC/ha/yr) |-------(m2/m2)-------|              N/ha'
   end if
   
   write (File_no(2),'( 3(f6.1,a), 2(f5.1,a), 4(f5.1,a), 1(f6.2,a), 1(i3,a), 1(f7.1,a) )') &
   mass_wood     ,',', &
   mass_grass    ,',', &
   mass_soil     ,',', &
   gpp           ,',', &
   npp           ,',', &
   lai_max_wood  ,',', &
   lai_max_grass ,',', &
   lai_mean_wood ,',', &
   lai_mean_grass,',', &
   canopy_cond   ,',', &
   biome         ,',', &
   density              
   
END SUBROUTINE output_annual



!**************************************************************************************************
! Output forest 3D structure
!**************************************************************************************************
SUBROUTINE output_forest &
      (year, height, bole, dbh_heartwood, dbh_sapwood, crown_diameter, &
      bole_x, bole_y, crown_x, crown_y, pft, tree_exist)

USE data_structure
implicit none

!Define arguments
integer,intent(IN)::year                               !year (1-Simulation_year)
integer,dimension(1:Max_no),intent(IN)::height         !tree height from 1.3m above ground (STEP)
integer,dimension(1:Max_no),intent(IN)::bole           !bole length (STEP)
real   ,dimension(1:Max_no),intent(IN)::dbh_heartwood  !heartwood diameter at 1.3m   (m)
real   ,dimension(1:Max_no),intent(IN)::dbh_sapwood    !sapwood diameter at 1.3m   (m)
real   ,dimension(1:Max_no),intent(IN)::crown_diameter !crown diameter (m)
real   ,dimension(1:Max_no),intent(IN)::bole_x         !x_location in forest stand (m)
real   ,dimension(1:Max_no),intent(IN)::bole_y         !y_location in forest stand (m)
real   ,dimension(1:Max_no),intent(IN)::crown_x        !x_location of crown (m)
real   ,dimension(1:Max_no),intent(IN)::crown_y        !y_location of crown (m)
integer,dimension(1:Max_no),intent(IN)::pft            !Plant Functional Type (1-PFT_no)
logical,dimension(1:Max_no),intent(IN)::tree_exist     !tree exist or not (.TRUE. or .FALSE.)

!Define local variable
integer count, i

!Count tree number
   count = 0
   Do i=1, Max_no
      if ( tree_exist(i) ) count = count +1
   End do
   
!Write data
   write (File_no(3),'(i4,a1)') count,','
   Do i=1, Max_no
   if (tree_exist(i)) then
      write (File_no(3),'( 4(f6.1,a), 2(f7.3,a), 2(f7.4,a), i2,a )') &
      bole_x(i)                           ,',',& !1: bole location x
      bole_y(i)                           ,',',& !2: bole location y
      crown_x(i)                          ,',',& !3: crown location x
      crown_y(i)                          ,',',& !4: crown location y
      real(bole(i)) * STEP + 1.3          ,',',& !5: bole height
      real( height(i) - bole(i) ) * STEP  ,',',& !6: foliage height
      dbh_heartwood(i) + dbh_sapwood(i)   ,',',& !7: dbh
      crown_diameter(i)/2.0               ,',',& !8: crown_radius
      pft(i)                              ,','   !9: pft
      
   end if
   End do
   
   !For separate files
   !character :: file_name*32
   !write(file_name,*) year
   !file_name = trim(file_name) // '.txt'
   !open (File_no(3), file=file_name)
   !close (File_no(3))
   
END SUBROUTINE output_forest



!**************************************************************************************************
! Output air variables
!**************************************************************************************************
SUBROUTINE output_air &
      (year, doy, ap, vp, vpd, dnsa, vp_sat, slope_vps)

USE data_structure
implicit none

!Define arguments
integer,intent(IN)::year   !year (1-Simulation_year)
integer,intent(IN)::doy    !day of year (1-365)
real,intent(IN)::ap        !air pressure (hPa)
real,intent(IN)::vp        !vapour pressure (hPa)
real,intent(IN)::vpd       !vapour pressure  deficit (hPa)
real,intent(IN)::dnsa      !density of air (kg m-3)
real,intent(IN)::vp_sat    !saturation vapour pressure (hPa)
real,intent(IN)::slope_vps !slope of saturation vapour pressure related to tempertaure (hPa deg C-1)
!Write title
   if (year == 1 .and. doy==1) then
      write ( File_no(4),* ) '  Yr  doy  ap    vps      vp      vpd    dnsa  slope_vps'
   end if
   
!Write data
   write (File_no(4),'(2i4, 1f8.1, 5f8.3, f8.2)') &
      year      ,&   !integer
      doy       ,&   !integer
      
      ap        ,&   !real
      
      vp_sat    ,&   !real
      vp        ,&   !real
      vpd       ,&   !real
      
      dnsa      ,&   !real
      slope_vps      !real

END SUBROUTINE output_air



!**************************************************************************************************
! Output climate variables
!**************************************************************************************************
SUBROUTINE output_climate &
      (year, doy, tmp_air, tmp_soil, cloud, prec, humid, wind )

USE data_structure
implicit none

!Define arguments
integer,intent(IN)::year
integer,intent(IN)::doy

real,intent(IN)::tmp_air   ! 2m air temperature (Celcius)
real,intent(IN)::tmp_soil  ! soil temperature at 10 cm  depth (Celcius)
real,intent(IN)::cloud     ! total cloudness (fraction)
real,intent(IN)::prec      ! precipitation (mm day-1)
real,intent(IN)::humid     ! specific humidity (kg kg-1)
real,intent(IN)::wind      ! wind velocity (m s-1)

!Write title
   if (year == 1 .and. doy==1) then
      write ( File_no(5),* ) '  yr  doy    air   soil  cloud    prec   humid   wind'
   end if
   
!Write data
   write (File_no(5),'(2i5, 3f7.1, f8.2, f8.3, f7.1)') &
   year        ,& !year
   doy         ,& !day of the year
   tmp_air     ,& !2m air temperature (Celcius)
   tmp_soil    ,& !soil temperature at 10 cm  depth (Celcius)
   cloud       ,& !total cloudness (fraction)
   prec        ,& !precipitation (mm day-1)
   humid       ,& !specific humidity (kg kg-1)
   wind           !wind  velocity (m s-1)
   
END SUBROUTINE output_climate



!**************************************************************************************************
! Output air variables
!**************************************************************************************************
SUBROUTINE output_radiation &
      (year, doy, sl_hgt, dlen, rad, par, par_grass)

USE data_structure
implicit none

!Define arguments
integer,intent(IN)::year   !year (1-Simulation_year)
integer,intent(IN)::doy    !day of year (1-365)
real,intent(IN)::sl_hgt    !solar hight at midday (degree)
real,intent(IN)::dlen      !day length (hour)
real,intent(IN)::rad       !global radiation at the ground surface (W/m2)
real,intent(IN)::par       !canopy top PAR of mid-day (micro mol photon m-2 s-1)
real,intent(IN)::par_grass !under canopy PAR of mid-day (micro mol photon m-2 s-1)

!Write title
   if (year == 1 .and. doy==1) then
      write ( File_no(6),* ) ' Yr doy  s_hgt dlen  rad    par     par_grass'
   end if
   
!Write data
   write (File_no(6),'(2i4, 2f6.1, 1f7.1, 2f8.1)') &
      year      ,& !integer
      doy       ,& !integer
      sl_hgt    ,& !real (degree)
      dlen      ,& !real (hour)
      rad       ,& !real (W/m2)
      par       ,& !real (micro mol photon m-2 s-1)
      par_grass    !real (micro mol photon m-2 s-1)

END SUBROUTINE output_radiation



!**************************************************************************************************
! Output net radiation
!**************************************************************************************************
SUBROUTINE output_netradiation &
      (year, doy, albedo_soil, albedo_veg, &
      radnet_soil, radnet_veg)

USE data_structure
implicit none

!Define arguments
integer,intent(IN)::year     !year (1-Simulation_year)
integer,intent(IN)::doy      !day of year (1-365)
real,intent(IN)::albedo_soil !soil surface albedo
real,intent(IN)::albedo_veg  !vegetation area albedo
real,intent(IN)::radnet_soil !soil surface net radiation, short wave (W m-2)
real,intent(IN)::radnet_veg  !canopy net radiation, short wave (W m-2)

!Write title
   if (year == 1 .and. doy==1) then
      write ( File_no(7),* ) ' Yr doy  s_alb  v_alb  radnet_soil radnet_veg'
   end if
   
!Write data
   write (File_no(7),'(2i4, 2f7.2, 2f10.1)') &
      year        ,&   !integer
      doy         ,&   !integer
      
      albedo_soil ,&   !real
      albedo_veg  ,&   !real
      radnet_soil ,&   !real
      radnet_veg       !real

END SUBROUTINE output_netradiation



!**************************************************************************************************
! Output water status
!**************************************************************************************************
SUBROUTINE output_water &
      (year, doy, pool_snow, pool_w1, pool_w2, W_sat)

USE data_structure
implicit none

!Define arguments
integer,intent(IN)::year      !year (1-Simulation_year)
integer,intent(IN)::doy       !day of year (1-365)
real   ,intent(IN)::pool_w1   !soil water content of soil layer 1 (mm)
real   ,intent(IN)::pool_w2   !soil water content of soil layer 2 (mm)
real   ,intent(IN)::pool_snow !water equivalent snow depth (mm)
real   ,intent(IN)::W_sat     !saturate point   (m3/m3, 0.0 -> 1.0)

!Define local valuable
real a1, a2

!Prepare values
a1 = pool_w1 / (W_sat * Depth1)
a2 = pool_w2 / (W_sat * Depth2)

!Write title
   if (year == 1 .and. doy==1) then
      write ( File_no(8),* ) 'Yr doy   snow   pool_w1   pool_w2  ms_per_whc1   ms_per_whc2'
   end if

!Write data
   write (File_no(8),'(2i4, 5f10.2, 2f10.3)') &
      year      ,&   !integer
      doy       ,&   !integer
      pool_snow ,&
      pool_w1   ,&
      pool_w2   ,&
      a1        ,&
      a2          

END SUBROUTINE output_water



!**************************************************************************************************
! Output carbon flux status
!**************************************************************************************************
SUBROUTINE output_cflux &
          (year, doy, c_uptake, c_emission, pool_litter, pool_som_int, pool_som_slow, &
           mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
           gmass_leaf, gmass_root, gmass_available, gmass_stock, type_grass)

USE data_structure
implicit none

!Define arguments
integer,intent(IN)::year       !year  (1-Simulation_year)
integer,intent(IN)::doy        !day of year (1-365)
real   ,intent(IN)::c_emission !monthly carbon emission (C g /month/stand)
real   ,intent(IN)::c_uptake   !monthly carbon uptake   (C g /month/stand)

real   ,intent(IN)::pool_litter   !Litter in a stand (g drymatter / stand)
real   ,intent(IN)::pool_som_int  !SOM with intermediated decomposition rate (g dm/stand)
real   ,intent(IN)::pool_som_slow !SOM with slow decomposition rate (g dm/stand)

real,dimension(1:Max_no),intent(IN)::mass_leaf       !biomass of foliage (g DM)
real,dimension(1:Max_no),intent(IN)::mass_trunk      !biomass of stem    (g DM)
real,dimension(1:Max_no),intent(IN)::mass_root       !biomass of root    (g DM)
real,dimension(1:Max_no),intent(IN)::mass_stock      !biomass for stock  (g DM)
real,dimension(1:Max_no),intent(IN)::mass_available  !biomass available  (g DM)

real,dimension(1:2)     ,intent(IN)::gmass_leaf      !biomass of grass foliage   (g DM/m2)
real,dimension(1:2)     ,intent(IN)::gmass_root      !biomass of grass root      (g DM/m2)
real,dimension(1:2)     ,intent(IN)::gmass_available !biomass of grass available (g DM/m2)
real,dimension(1:2)     ,intent(IN)::gmass_stock     !biomass of grass stock     (g DM/m2)
integer                 ,intent(IN)::type_grass      !grass type priority (3:C3, 4:C4)

!Define local valiables
real a1, a2, a3
real b1, b2, b3
real c1, c2

!Prepare output values
   a1 = c_uptake   /Max_loc/Max_loc / 100.0 !carbon uptake (g C/month/m2)
   a2 = c_emission /Max_loc/Max_loc / 100.0 !carbon uptake (g C/month/m2)
   a3 = a1 - a2                             !NEP           (g C/month/m2)
   
   b1 = pool_litter   * C_in_drymass / Max_loc / Max_loc / 100.0
   b2 = pool_som_int  * C_in_drymass / Max_loc / Max_loc / 100.0
   b3 = pool_som_slow * C_in_drymass / Max_loc / Max_loc / 100.0
   
   c1 = sum(mass_leaf) + sum(mass_trunk) + sum(mass_root) + sum(mass_stock) + sum(mass_available)
   c1 = c1 * C_in_drymass / Max_loc / Max_loc / 100.0
   
   if (type_grass==3) then
   c2 = 0.9 * (gmass_leaf(1)+gmass_root(1)+gmass_available(1)+gmass_stock(1)) + &
        0.1 * (gmass_leaf(2)+gmass_root(2)+gmass_available(2)+gmass_stock(2))    
   else
   c2 = 0.1 * (gmass_leaf(1)+gmass_root(1)+gmass_available(1)+gmass_stock(1)) + &
        0.9 * (gmass_leaf(2)+gmass_root(2)+gmass_available(2)+gmass_stock(2))    
   end if
   c2 = c2 * C_in_drymass / 100.0

!Write title
if (year == 1 .and. Month(doy)==1) then
   write (File_no(9),*) &
         '       |----- Unit = kg C ha-1 -------------------------------------->|'
   write (File_no(9),*) &
         'Yr  mon uptake  emiss    NEP   litter  SOM_int SOM_slow  mass_w mass_g'
end if

!Write data
write (File_no(9),'( 1(i4,a), 1(i2,a), 3(f6.1,a), 4(f8.1,a), 1(f6.1) )') &
      year       ,',', &
      Month(doy) ,',', &
      a1         ,',', &
      a2         ,',', &
      a3         ,',', &
      b1         ,',', &
      b2         ,',', &
      b3         ,',', &
      c1         ,',', &
      c2                

END SUBROUTINE output_cflux



!**************************************************************************************************
! Output grass
!**************************************************************************************************
SUBROUTINE output_grass &
      (year, doy, phenology, gpp, npp, C3g_no, C4g_no, &
      pool_litter, pool_som_int, pool_som_slow, &
      type_grass, gmass_leaf, gmass_root, gmass_available, &
      gmass_stock, lai_grass, lai_opt_grass)

USE data_structure
implicit none

!Define arguments
integer,intent(IN)::year !year (1-Simulation_year)
integer,intent(IN)::doy  !day of year (1-365)

real,intent(IN)::pool_litter   !litter pool in a stand (g drymatter / stand)
real,intent(IN)::pool_som_int  !pool of s.o.m. with intermediated decomposition rate (g dm/stand)
real,intent(IN)::pool_som_slow !pool of s.o.m. with slow decomposition rate (g dm / stand)

logical,dimension(PFT_no),intent(IN)::phenology !Phenological status (T:foliation, F:dormance)
real   ,dimension(PFT_no),intent(IN)::gpp       !daily GPP for each pft (g drymatter m-2 day-1)
real   ,dimension(PFT_no),intent(IN)::npp       !daily NPP for each pft (g drymatter m-2 day-1)

integer,intent(IN)::C3g_no !PFT number of C3 grass
integer,intent(IN)::C4g_no !PFT number of C4 grass

integer            ,intent(IN)::type_grass      !grass type priority (3:C3 type, 4:C4 type)
real,dimension(1:2),intent(IN)::gmass_leaf      !grass foliage    biomass (g/m2)
real,dimension(1:2),intent(IN)::gmass_root      !grass root       biomass (g/m2)
real,dimension(1:2),intent(IN)::gmass_available !grass available  biomass (g/m2)
real,dimension(1:2),intent(IN)::gmass_stock     !grass stock      biomass (g/m2)
real,dimension(1:2),intent(IN)::lai_grass       !leaf area index of grass   (m2/m2)
real,dimension(1:2),intent(IN)::lai_opt_grass   !optimum LAI of grass layer (m2/m2)

!Define local variable
real      a1, a2
character phenology_symbol_c3, phenology_symbol_c4
real      mass_f_ave, mass_r_ave, mass_stock_ave
real      mass_total_ave, mass_soil_ave, lai_ave, lai_opt_ave

!set phenology_symbol
phenology_symbol_c3='-' ; if (phenology(C3g_no)) phenology_symbol_c3='*'
phenology_symbol_c4='-' ; if (phenology(C4g_no)) phenology_symbol_c4='*'

!prepare output variables
  a1 = gmass_leaf(1) + gmass_root(1) + gmass_available(1) + gmass_stock(1)
  a2 = gmass_leaf(2) + gmass_root(2) + gmass_available(2) + gmass_stock(2)
  
if (type_grass==3) then
  mass_total_ave = 0.9*a1              + 0.1*a2
  mass_f_ave     = 0.9*gmass_leaf(1)   + 0.1*gmass_leaf(2)
  mass_r_ave     = 0.9*gmass_root(1)   + 0.1*gmass_root(2)
  mass_stock_ave = 0.9*gmass_stock(1)  + 0.1*gmass_stock(2)
  lai_ave        = 0.9*lai_grass(1)    + 0.1*lai_grass(2)
  lai_opt_ave    = 0.9*lai_opt_grass(1)+ 0.1*lai_opt_grass(2)
else
  mass_total_ave = 0.1*a1              + 0.9*a2
  mass_f_ave     = 0.1*gmass_leaf(1)   + 0.9*gmass_leaf(2)
  mass_r_ave     = 0.1*gmass_root(1)   + 0.9*gmass_root(2)
  mass_stock_ave = 0.1*gmass_stock(1)  + 0.9*gmass_stock(2)
  lai_ave        = 0.1*lai_grass(1)    + 0.9*lai_grass(2)
  lai_opt_ave    = 0.1*lai_opt_grass(1)+ 0.9*lai_opt_grass(2)
end if
  
  mass_total_ave = C_in_drymass * mass_total_ave
  mass_soil_ave  = C_in_drymass * (pool_litter+pool_som_int+pool_som_slow) / Max_loc / Max_loc

!Write title
if (year==1 .and. doy==1) then
   write ( File_no(12),* ) 'Yr doy pt C3   F    R    S  lai lai_o  C4   F    R    S  lai lai_o'
end if

!Write data
   write (File_no(12),'( i3 , i4 , i3 , 2(a3, 3f5.0, 2f5.1) )') &
    year, doy, type_grass,  &
    
    phenology_symbol_c3, gmass_leaf(1), gmass_root(1), gmass_stock(1), &
    lai_grass(1), lai_opt_grass(1), &
    
    phenology_symbol_c4, gmass_leaf(2), gmass_root(2), gmass_stock(2), &
    lai_grass(2), lai_opt_grass(2)

END SUBROUTINE output_grass



!**************************************************************************************************
! Biomass composition
!**************************************************************************************************
SUBROUTINE output_biomass &
        (mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, tree_exist, pft, &
        gmass_leaf, gmass_root, gmass_stock, gmass_available, type_grass, C3g_no, C4g_no )

USE data_structure
implicit none

!*** Define arguments ***
real   ,dimension(1:Max_no),intent(IN)::mass_leaf      !biomass of foliage (g DM)
real   ,dimension(1:Max_no),intent(IN)::mass_trunk     !biomass of stem    (g DM)
real   ,dimension(1:Max_no),intent(IN)::mass_root      !biomass of root    (g DM)
real   ,dimension(1:Max_no),intent(IN)::mass_stock     !biomass for stock  (g DM)
real   ,dimension(1:Max_no),intent(IN)::mass_available !biomass available  (g DM)
integer,dimension(1:Max_no),intent(IN)::pft            !Plant Functional Type (1-PFT_no)
logical,dimension(1:Max_no),intent(IN)::tree_exist     !tree exist flag

real,dimension(1:2),intent(IN)::gmass_leaf      !grass foliage   biomass (g DM/m2)
real,dimension(1:2),intent(IN)::gmass_root      !grass root      biomass (g DM/m2)
real,dimension(1:2),intent(IN)::gmass_stock     !grass stock     biomass (g DM/m2)
real,dimension(1:2),intent(IN)::gmass_available !grass available biomass (g DM/m2)
integer            ,intent(IN)::type_grass      !grass type priority (3:C3, 4:C4)
integer            ,intent(IN)::C3g_no          !PFT number of C3 grass
integer            ,intent(IN)::C4g_no          !PFT number of C4 grass

!*** Define local variables ***
real,dimension(PFT_no)::biomass !biomass for each PFT (g C / m2)
integer                 no      !for loop counter
real                    x, y    !for gengeral usage

!*** reset output variables ***
   biomass(:)=0.0
   
!*** woody biomass (g dm / stand) ***
   Do no=1, Max_no
   If (tree_exist(no)) then
      biomass(pft(no)) = biomass(pft(no))+&
      mass_leaf(no)+mass_trunk(no)+mass_root(no)+mass_stock(no)+mass_available(no)
   End if
   End do
   
!*** grass biomass (g dm / stand) ***
   x = gmass_leaf(1) + gmass_root(1) + gmass_stock(1) + gmass_available(1)
   y = gmass_leaf(2) + gmass_root(2) + gmass_stock(2) + gmass_available(2)
   
   if (type_grass==3) then
      biomass(C3g_no) = 0.9 * Max_loc * Max_loc * x
      biomass(C4g_no) = 0.1 * Max_loc * Max_loc * y
   else
      biomass(C3g_no) = 0.1 * Max_loc * Max_loc * x
      biomass(C4g_no) = 0.9 * Max_loc * Max_loc * y
   endif
   
!*** adjust unit ( g dm / stand ---> Mg C / ha ) ***
   biomass(:) = biomass(:) * C_in_drymass / Max_loc / Max_loc / 100.0
   
!*** write title and data ***
   write (File_no(13),'( 10(f6.1,1x) )') biomass(:)
   
END SUBROUTINE output_biomass
