!*************************************************************************************************
! Start up procedures for selected sites
! (read common parameters and climate data)
!*************************************************************************************************

   Include 'module.f90'         
   Include 'main.f90'           
   Include 'initialize.f90'     
   Include 'metabolic.f90'      
   Include 'output.f90'         
   Include 'physics.f90'        
   Include 'population_regu.f90'
   Include 'spacial_calc.f90'   
   Include 'etc.f90'            

PROGRAM start_point

USE data_structure
implicit none
common PN_s, PN_r

!*************************    Set variables    ***********************
!Climate data
real,dimension(365)::tmp_air   !2m air temperature (Celcius)
real,dimension(365)::tmp_soil  !soil temperature at 10 cm  depth (Celcius)
real,dimension(365)::cloud     !total cloudiness (fraction)
real,dimension(365)::prec      !precipitation (mm day-1)
real,dimension(365)::humid     !specific humidity (kg kg-1)
real,dimension(365)::wind      !wind velocity (m s-1)

!Location data
integer Mask         !Land ocean mask (1:land, 0:ocean)
real    ALT          !altitude (m above MSL)
real    Albedo_soil0 !albedo, default
real    W_fi         !filed capacity   (m3/m3, 0.0 -> 1.0)
real    W_wilt       !wilting point    (m3/m3, 0.0 -> 1.0)
real    W_sat        !saturate point   (m3/m3, 0.0 -> 1.0)
real    W_mat        !matrix potential (m, -0.0001 -> -3.0)

!PFT specific constants for commom use
real,dimension(PFT_no)::PN_s !proportion of nitrogen:stem (gN / gBiomass)
real,dimension(PFT_no)::PN_r !proportion of nitrogen:root (gN / gBiomass)

!Define local variables
real LAT       !latitude for simulate
real LON       !logitude for simulate

integer Fn_out !output file number
integer point  !point number @ normal coodinate
integer i      !for loop counter

!***********************  set NameList  *************************
    namelist /Control/        &
        Simulation_year, Lon_start, Lon_end, Lat_start, Lat_end, &
        Max_loc, Dived, Max_hgt, Max_no, Depth1, Depth2, Depth3, STEP, &
        I_seed, C_in_drymass, File_no, Outputting, Logging, &
        Fn_climate, Fn_location
    
    namelist /PFT_type/       &
        Life_type, Phenology_type
    
    namelist /Respiration/    &
        RM, RG_f, RG_s, RG_r, RG_stock_in, RG_stock_out
    
    namelist /Turnover_n/       &
        TO_f,  TO_s,  TO_r
    
    namelist /Metabolic/      &
        ALM1, ALM2, ALM3, ALM4, &
        CD_max, LA_max, FR_ratio, HGT_max, HGT_s, P_root, P_ic, SLA, PN_f, Pmax
    
    namelist /Assimilation/ &
        EK0, Lue0, Topt0, Tmin, Tmax, &
        GS_b1, GS_b2, GS_b3, KM, CO2cmp0, Kwater
    
    namelist /Dynamics/       &
        M1, M2, TC_min, Heat_sensitive, Heat_base, &
        P_establish, TC_max, GDD_min, PAR_min, AGE_max
    
    namelist /Disturbance/    &
        Fuel_min, Moisture_extinction, M3
    
    namelist /Soil_resp/  &
        TO_fast, TO_slow, F_air, F_inter, F_resp, F_fast
    
    namelist /Soil_hydrology/ &
        K0_infilt, K1_u_up,   K2_u_up,   K1_s_up,   K2_s_up,  &
                   K1_u_low,  K2_u_low,  K1_s_low,  K2_s_low, &
                   K1_u_base, K2_u_base, K1_s_base, K2_s_base  

!***********************  set location  *************************
!north:+, south:- (decimalized)
   LAT      = 32.01
!west :-, east :+ (decimalized)
   LON     = 130.52

!set cordinate variables
   point = (90-int(LAT)-1)*360 + int(LON+180) + 1

!***********************  Prepare input data  *************************
!Parameter files
   open (1, file='parameter.txt', action='READ')
      read ( unit=1, nml=Control)       
      read ( unit=1, nml=PFT_type)      
      read ( unit=1, nml=Respiration)   
      read ( unit=1, nml=Turnover_n)    
      read ( unit=1, nml=Metabolic)     
      read ( unit=1, nml=Assimilation)  
      read ( unit=1, nml=Dynamics)      
      read ( unit=1, nml=Disturbance)   
      read ( unit=1, nml=Soil_resp)     
      read ( unit=1, nml=Soil_hydrology)
   close (1)
   
   !nitrogen contents per biomass for each PFT (gN / gBiomass)
    do i = 1, PFT_no
        PN_s(i) = 0.145 * PN_f(i) !@sapwood
        PN_r(i) = 0.86  * PN_f(i) !@root
    end do

!Location data
    write (*,*) 'reading location data...'
    
    !mask
    open ( File_no(1), file=Fn_location(1))
    do i=1, point; read(File_no(1),*) Mask; end do
    close( File_no(1) )
    
    !altitude
    open ( File_no(1), file=Fn_location(2))
    do i=1, point; read(File_no(1),*) ALT; end do
    close( File_no(1) )
    
    !albedo of soil
    open ( File_no(1), file=Fn_location(3))
    do i=1, point; read(File_no(1),*) Albedo_soil0; end do
    close( File_no(1) )
    
    !field capacity
    open ( File_no(1), file=Fn_location(4))
    do i=1, point; read(File_no(1),*) W_fi; end do
    close( File_no(1) )
    
    !saturate point
    open ( File_no(1), file=Fn_location(5))
    do i=1, point; read(File_no(1),*) W_sat; end do
    close( File_no(1) )
    
    !matrix potential
    open ( File_no(1), file=Fn_location(6))
    do i=1, point; read(File_no(1),*) W_mat; end do
    close( File_no(1) )
    W_mat = -1.00*W_mat
    
    !wilting point
    open ( File_no(1), file=Fn_location(7))
    do i=1, point; read(File_no(1),*) W_wilt; end do
    close( File_no(1) )
    
    !data processing
    if (W_fi > W_sat ) W_fi = W_sat
    if (W_wilt  > W_sat ) W_wilt  = W_sat
    
    !checker
    if ( Mask         == 0  .or. &
        ALT          <= 0.0 .or. &
        Albedo_soil0 <= 0.0 .or. &
        W_sat        <= 0.0 .or. &
        W_fi         <= 0.0 .or. &
        W_mat        <= 0.0 .or. &
        W_wilt       <= 0.0        ) STOP

!Climatic data
    write (*,*) 'reading climate data...'
    
    !2m air temperature (Celcius)
    open ( File_no(1), file=Fn_climate(1))
    read ( File_no(1), *)( tmp_air(i), i=1, 365 )
    close( File_no(1) )
    
    !soil temperature at 10cm depth (Celcius)
    open ( File_no(1), file=Fn_climate(2))
    read ( File_no(1), *)( tmp_soil(i), i=1, 365 )
    close( File_no(1) )
    
    ! total  cloudiness,  fraction
    open ( File_no(1), file=Fn_climate(3))
    read ( File_no(1), *)( cloud(i), i=1, 365 )
    close( File_no(1) )
    
    ! precipitation,  mm day-1
    open ( File_no(1), file=Fn_climate(4))
    read ( File_no(1), *)( prec(i), i=1, 365)
    close( File_no(1) )
    
    ! specific  humidity,  kg  kg-1
    open ( File_no(1), file=Fn_climate(5))
    read ( File_no(1), *)( humid(i), i=1, 365)
    close( File_no(1) )
    
    ! wind  velocity (m s-1)
    open ( File_no(1), file=Fn_climate(6))
    read ( File_no(1), *)( wind(i), i=1, 365 )
    close( File_no(1) )
    
!***********************  Simulation  *************************
     
      !location
      write (*,*)
      write (*,*) '*********  Coodinate Configurations  *********'
      write (*,*) 'Latitude, Longtitude   :', LAT, LON
      write (*,*) 'Point nomal            :', point
      write (*,*)
      
      !soil information
      write (*,*) '*********  Location properties  *********'
      write (*,*) 'Al;titude   :', ALT         
      write (*,*) 'Albedo_soil0:', Albedo_soil0
      write (*,*) 'W_sat       :', W_sat       
      write (*,*) 'W_fi        :', W_fi        
      write (*,*) 'W_mat       :', W_mat       
      write (*,*) 'W_wilt      :', W_wilt      
      write (*,*)                              
      
      !climate information
      write (*,*) '*********  Wether statistics (annual mean)  *********'
      write (*,*) '2m air temperature  (Cecius)  :', sum(tmp_air (:)) / 365.0
      write (*,*) 'soil temperature    (Cecius)  :', sum(tmp_soil(:)) / 365.0
      write (*,*) 'precipitation       (mm/year) :', sum(prec    (:))        
      write (*,*) 'cloud               (fraction):', sum(cloud   (:)) / 365.0
      write (*,*) 'humidity            (kg/kg)   :', sum(humid   (:)) / 365.0
      write (*,*) 'wind                (m/s)     :', sum(wind    (:)) / 365.0
      write (*,*) 
     
      write (*,*) '*********  Now simulating  *********'
      write (*,*) 
     Call main ( &
     LAT, LON, &
     tmp_air(:), tmp_soil(:), cloud(:), prec(:), humid(:), wind(:), &
     ALT, Albedo_soil0, W_fi, W_wilt, W_sat, W_mat )
     
      write (*,*) '*********  Done  *********'

STOP

END PROGRAM start_point
